/*********************************************
Project : ATmega128_MT
Chip type           : ATmega128
Program type        : Application
Clock frequency     : 16,000000 MHz
*********************************************/

#include <mega128.h>
#include <delay.h>          

#define		BIT6				0x40  
#define		LCD_CHAR_SPEED			2
#define		timeout				250                                                                              


#define		B1				PINA.0  
#define		B2				PINA.1
#define		B3				PINA.2
#define		B4				PINA.3
#define		B5				PINA.4                      
#define		BUZZ1				PORTE.4		//BUZZ OUTPUT
#define		BUZZ2				PORTE.5		//BUZZ OUTPUT
#define		ON				1
#define     	OFF				0      
#define 	RELAY				PORTA.6
#define		RS				PORTC.0
#define		R_W				PORTC.1
#define		E				PORTC.2			
#define		TXD				PORTD.3
#define		RXD				PIND.2	
#define		DALLAS				PINA.5

#define		DISP_ON				0b00001100	//LCD control constants
#define		DISP_OFF			0b00001000	//
#define		CLR_DISP			0b00000001	//
#define		CUR_HOME			0b00000010	//
#define		ENTRY_INC			0b00000110	//
#define		DD_RAM_ADDR			0b10000000	//
#define		DD_RAM_ADDR2			0b11000000	//  
#define		DD_RAM_ADDR3			40		//      
#define		CG_RAM_ADDR			0b01000000	//      

const unsigned char Message[] = "   AVR-MT-128    olimex.com/dev ";
const unsigned char MessageDLS[] = " DALLAS PRESENT ";
const unsigned char MessageTMR1[] = "TMR1 is CLOCKED ";   
const unsigned char MessageTMR2[] = "TMR2 is CLOCKED ";
const unsigned char MessageRS232[] = " olimex.com/dev ";  
const unsigned char MessageALABALA[] = " Visit the site  www.olimex.com ";
const unsigned char MessageSend_to_RS232[] = "sending to RS232";
unsigned char i,counter, receiv_data,data,temp,flag_rs,status_main,dls;     

#define 	RXB8 		1
#define 	TXB8 		0
#define 	UPE 		2
#define 	OVR 		3
#define 	FE 		4
#define 	UDRE 		5
#define 	RXC 		7

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<OVR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

//-------------------- LCD ini after reset-------------
void BUZZER(void)
{
  while (B4==0)
     {
      	BUZZ1=OFF;			//250us
      	BUZZ2=ON;
      	delay_us(125);
      	BUZZ2=OFF;
      	BUZZ1=ON;
      	delay_us(125);
     }
}                                      

void _E (void)
{
	E=1;
	delay_us(5);
	E=0;
}	


void LCD_Ini(void)
{
	RS=0;				
	delay_ms(110);	
	PORTC=0b00110000;		
	_E();	
	delay_ms(10);	
	PORTC=0b00110000;
	_E();
	delay_ms(10);
	PORTC=0b00110000;
	_E();
	delay_ms(10);
	PORTC=0b00100000;
	_E();
}      
    
//--------Send char to LCD-----------------------------
SEND_CHAR (unsigned char c)
{    
	delay_ms(LCD_CHAR_SPEED);              	    //Delay for LCD char
	data = 0b00001111 | c;			    //get upper nibble
	PORTC = (PORTC | 0b11110000) & data;        //set D4-D7
	RS=1;		  			    //set LCD to data mode
	_E();
	data = c << 4;				    //get down nibble
	PORTC = (PORTC & 0b00001111) | data;	    //set D4-D7 (only PORTC4-PORTC7)
	RS=1;					    //set LCD to data mode
	_E();
}               
    
//--------Send command to LCD-----------------------------
SEND_CMD (unsigned char c)
{    
	delay_ms(LCD_CHAR_SPEED);              	    //Delay for LCD char
	data = 0b00001111 | c;			    //get upper nibble
	PORTC = (PORTC | 0b11110000) & data;        //set D4-D7
	RS=0;					    //set LCD to data mode
	_E();    
	data = c << 4;				    //get down nibble
	PORTC = (PORTC & 0b00001111) | data;	    //set D4-D7 (only PORTC4-PORTC7)
	RS=0;					    //set LCD to data mode
	_E();
}                    

unsigned char getchar1(void)
{
char status,data;
while (1)
      {
      while (((status=UCSR1A) & RX_COMPLETE)==0);
      data=UDR1;         
      if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)   return data;               
                                
      };
} 

void putchar1(char c)
{
while ((UCSR1A & DATA_REGISTER_EMPTY)==0);
UDR1=c;
} 

#define ADC_VREF_TYPE 0x40

unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input|ADC_VREF_TYPE;
ADCSRA|=0x40;
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}
  
//----------------------------------------------------------------



void main(void)
{     

// Input/Output Ports initialization
// Port A initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=Out Func7=In 
// State0=T State1=T State2=T State3=T State4=T State5=T State6=0 State7=T 
PORTA=0x00;
DDRA=0x40;

// Port B initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
PORTB=0x00;
DDRB=0x00;

// Port C initialization
// Func0=Out Func1=Out Func2=Out Func3=In Func4=Out Func5=Out Func6=Out Func7=Out 
// State0=0 State1=0 State2=0 State3=T State4=0 State5=0 State6=0 State7=0 
PORTC=0x00;
DDRC=0xF7;

// Port D initialization
// Func0=In Func1=In Func2=In Func3=Out Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=T State2=T State3=0 State4=T State5=T State6=T State7=T 
PORTD=0x00;
DDRD=0x08;

// Port E initialization
// Func0=In Func1=In Func2=In Func3=In Func4=Out Func5=Out Func6=In Func7=In 
// State0=T State1=T State2=T State3=T State4=0 State5=0 State6=T State7=T 
PORTE=0x00;
DDRE=0x30;

// Port F initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
PORTF=0x00;
DDRF=0x00;

// Port G initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In 
// State0=T State1=T State2=T State3=T State4=T 
PORTG=0x00;
DDRG=0x00;


// Timer/Counter 1 initialization
// Clock source: T1 pin Rising Edge
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// OC1C output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x07;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
OCR1CH=0x00;
OCR1CL=0x00;

// Timer/Counter 2 initialization
// Clock source: T2 pin Rising Edge
// Mode: Normal top=FFh
// OC2 output: Disconnected
TCCR2=0x07;
TCNT2=0x00;
OCR2=0x00;


// USART1 initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART1 Receiver: On
// USART1 Transmitter: On
// USART1 Mode: Asynchronous
// USART1 Baud rate: 9600
UCSR1A=0x00;
UCSR1B=0x18;     		//(00011000)
UCSR1C=0x06;
UBRR1H=0x00;
UBRR1L=0x67;			//103

	delay_ms(100);               	               	
    	LCD_Ini(); 					//LCD initialisation after reset     
        delay_ms(10);  
		
//---------------------------------------------------

	SEND_CMD(DISP_ON);
	delay_ms(10);  						
	SEND_CMD(CLR_DISP); 
 	delay_ms(10);  
	SEND_CMD(DD_RAM_ADDR);                                   
	
	for (i=0 ; i!=32; i++)      			 //Send Message "AVR-MT olimex.com/dev" to LCD
	  {
		temp=Message[i];
		SEND_CHAR(temp);  
		if (i == 15) SEND_CMD(DD_RAM_ADDR2);
	  }
	SEND_CMD(DD_RAM_ADDR);	

while (1)
  {                
     if ((UCSR1A & 0x80) !=0 ) 
      {                      
	if (counter == 0 || counter == 32) 
          {
            SEND_CMD(CLR_DISP); 
     	    SEND_CMD(DD_RAM_ADDR); 
     	    counter=0;
          }						 
    	if(counter == 16) SEND_CMD(DD_RAM_ADDR2); 	//set second row of LCD	
        counter++; 
        receiv_data=UDR1;               
        SEND_CHAR(receiv_data);				//send to LCD
        putchar1(receiv_data+1);			//RS232 echo
      }  

//buttons scan                
    if (B1==0) RELAY = ON;        
    else RELAY = OFF;	      
    delay_ms(1);
    if (B2==0) 
      {
        SEND_CMD(CLR_DISP);      	
        SEND_CMD(DD_RAM_ADDR);
        counter=0;
      }  
    delay_ms(1);           
    if (B3==0)
      {       
         SEND_CMD(CLR_DISP);   
         SEND_CMD(DD_RAM_ADDR);   
         for (i=0 ; i!=32; i++)      				//Send Message "AVR-MT olimex.com/dev" to LCD
	   {
	      temp=MessageALABALA[i];
	      SEND_CHAR(temp);  
	      if (i == 15) SEND_CMD(DD_RAM_ADDR2);	      
	   }	                                  
	 counter=0;  
      }       
    delay_ms(1);       
    if(B4==0) BUZZER(); 
    delay_ms(1);
    if (B5==0) 
      {          
        SEND_CMD(CLR_DISP);   
        SEND_CMD(DD_RAM_ADDR);          
        for (i=0;i!=16;i++)
          {
	     temp=MessageRS232[i];	     
	     putchar1(temp);	      
	     data=MessageSend_to_RS232[i];  
	     SEND_CHAR(data);	     
          }     
        counter=0;     	
      }         
    delay_ms(1); 
//----------------DALLAS SCAN with time out------------------
//-----------------------------------------------------------

    if (DALLAS == 0) 
      {
    	dls=timeout;
	while(dls != 0)
	if (DALLAS == 0) dls--;			
	else dls=timeout;	       
	SEND_CMD(CLR_DISP); 
	SEND_CMD(DD_RAM_ADDR);                  
                   
//--------------Send to LCD----------------------------------
	for (i=0 ; i!=16; i++)
	  {
		temp=MessageDLS[i];      					
		SEND_CHAR(temp);							//Send Message " DALLAS PRESENT "
	  }		        
	counter = 0;                        	
      }      
//----------------TIMER1-------------------------------------
//-----------------------------------------------------------

		if(TCNT1L >= 2 || TCNT1H != 0)
		  {
			TCNT1L=0;
			TCNT1H=0;   		
			SEND_CMD(CLR_DISP);
			SEND_CMD(DD_RAM_ADDR);
//-------------Send to LCD----------------------------------
			for (i=0 ; i!=16; i++)
			  {
				temp=MessageTMR1[i];                			//Send Message "TMR1 is CLOCKED "
				SEND_CHAR(temp);
	   		  }	   	   	
	   		counter = 0;  	  
		  }          
//----------------TIMER2-------------------------------------
//-----------------------------------------------------------

		if(TCNT2 >= 2)
		  {
			TCNT2=0;				
			SEND_CMD(CLR_DISP);
			SEND_CMD(DD_RAM_ADDR2);
//-------------Send to LCD----------------------------------
			for (i=0 ; i!=16; i++)
			  {
				temp=MessageTMR2[i];                			//Send Message "TMR1 is CLOCKED "
				SEND_CHAR(temp);
	   		  }	   	   	
	   		counter = 0;  	  
		  }         		                 		                              
  }
}
